package net.minecraft.src;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

// SDK ModLoaderMp
public class Packet200ModLoader extends Packet {
	private static final int MAX_DATA_LENGTH = 65535;
	
    public Packet200ModLoader() { }
    
    @Override
    public void readPacketData(DataInputStream datainputstream) throws IOException
    {
    	// Read modId
        modId = datainputstream.readInt();
        
        // Read packetType
        packetType = datainputstream.readInt();
        
        // Read dataInt
        int dataIntSize = datainputstream.readInt();
        if(dataIntSize > MAX_DATA_LENGTH)
        	throw new IOException(String.format("Integer data size of {0} is higher than the max ({1}).", dataIntSize, MAX_DATA_LENGTH));
        dataInt = new int[dataIntSize];
        for(int i = 0; i < dataIntSize; i++)
        	dataInt[i] = datainputstream.readInt();
        
        // Read dataFloat
        int dataFloatSize = datainputstream.readInt();
        if(dataFloatSize > MAX_DATA_LENGTH)
        	throw new IOException(String.format("Float data size of {0} is higher than the max ({1}).", dataFloatSize, MAX_DATA_LENGTH));
        dataFloat = new float[dataFloatSize];
        for(int i = 0; i < dataFloatSize; i++)
        	dataFloat[i] = datainputstream.readFloat();
        
        // Read dataString
        int dataStringSize = datainputstream.readInt();
        if(dataStringSize > MAX_DATA_LENGTH)
        	throw new IOException(String.format("String data size of {0} is higher than the max ({1}).", dataStringSize, MAX_DATA_LENGTH));
        dataString = new String[dataStringSize];
        for(int i = 0; i < dataStringSize; i++)
        {
        	// Get length of string
        	int length = datainputstream.readInt();
        	if(length > MAX_DATA_LENGTH)
            	throw new IOException(String.format("String length of {0} is higher than the max ({1}).", length, MAX_DATA_LENGTH));
        	
        	// Get bytes
        	byte[] dataStringBytes = new byte[length];
        	datainputstream.read(dataStringBytes, 0, length);
        	
        	// Convert to string
        	dataString[i] = new String(dataStringBytes);
        }
    }
    
    @Override
    public void writePacketData(DataOutputStream dataoutputstream) throws IOException
    {
    	// Check maxes
        if(dataInt.length > MAX_DATA_LENGTH)
        	throw new IOException(String.format("Integer data size of {0} is higher than the max ({1}).", dataInt.length, MAX_DATA_LENGTH));
        if(dataFloat.length > MAX_DATA_LENGTH)
        	throw new IOException(String.format("Float data size of {0} is higher than the max ({1}).", dataFloat.length, MAX_DATA_LENGTH));
        if(dataString.length > MAX_DATA_LENGTH)
        	throw new IOException(String.format("String data size of {0} is higher than the max ({1}).", dataString.length, MAX_DATA_LENGTH));
    	
    	// Write modId
    	dataoutputstream.writeInt(modId);
    	
    	// Write packetType
    	dataoutputstream.writeInt(packetType);
    	
    	// Write dataInt
    	dataoutputstream.writeInt(dataInt.length);
    	for(int i = 0; i < dataInt.length; i++)
    		dataoutputstream.writeInt(dataInt[i]);
    	
    	// Write dataFloat
    	dataoutputstream.writeInt(dataFloat.length);
    	for(int i = 0; i < dataFloat.length; i++)
    		dataoutputstream.writeFloat(dataFloat[i]);
    	
    	// Write dataString
    	dataoutputstream.writeInt(dataString.length);
    	for(int i = 0; i < dataString.length; i++)
    	{
    		// Length for each string, then the string
    		dataoutputstream.writeInt(dataString[i].length());
    		dataoutputstream.writeBytes(dataString[i]);
    	}
    }
    
    @Override
    public void processPacket(NetHandler nethandler)
    {
        ModLoaderMp.HandleAllPackets(this);
    }
    
    @Override
    public int getPacketSize()
    {
    	// modId
        int packetSize = 1;
        
        // packetType
        packetSize++;
        
        // dataInt
        packetSize++;
    	packetSize += dataInt.length * Integer.SIZE;
    	
    	// dataFloat
    	packetSize++;
        packetSize += dataFloat.length * Float.SIZE;
        
        // dataString
        packetSize++;
        for(int i = 0; i < dataString.length; i++)
        {
        	packetSize++;
        	packetSize += dataString[i].length();
        }
        
        return packetSize;
    }
    
    public int modId;
    public int packetType;
    public int[] dataInt = new int[0];
    public float[] dataFloat = new float[0];
    public String[] dataString = new String[0];
}
